// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:meta/meta.dart';
import 'package:wyatt_form_bloc/src/enums/enums.dart';
import 'package:wyatt_form_bloc/src/form/form.dart';

part 'form_data_state.dart';

abstract class FormDataCubit extends Cubit<FormDataState> {
  FormDataCubit({required FormData inputs})
      : super(FormDataState(data: inputs));

  /// Change value of a field.
  /// 
  /// Inputs:
  /// - `field`: The key of the field to change.
  /// - `dirtyValue`: The new value of the field. (Wrapped in a dirty validator)
  void dataChanged(String field, FormInputValidator dirtyValue) {
    final _form = state.data.clone();

    if (_form.contains(field)) {
      _form.updateValidator(field, dirtyValue);
    } else {
      throw Exception('Form field $field not found');
    }

    emit(
      state.copyWith(
        data: _form,
        status: _form.validate(),
      ),
    );
  }

  /// Update entries list.
  /// 
  /// Inputs:
  /// - `data`: The new entries list.
  /// - `operation`: The operation to perform on the entries set.
  void updateFormData(
    FormData data, {
    SetOperation operation = SetOperation.replace,
  }) {
    FormData _form = data;

    switch (operation) {
      case SetOperation.replace:
        _form = data;
        break;
      case SetOperation.difference:
        _form = state.data.difference(data);
        break;
      case SetOperation.intersection:
        _form = state.data.intersection(data);
        break;
      case SetOperation.union:
        _form = state.data.union(data);
        break;
    }

    emit(
      state.copyWith(
        data: _form,
        status: _form.validate(),
      ),
    );
  }

  /// Submit the form.
  Future<void> submitForm();
}
